/**
  *  \file planetscentral/dbclient/dbset.h
  *  \brief Database Client: Sets
  */
#ifndef C2SERVER_PLANETSCENTRAL_DBCLIENT_DBSET_H
#define C2SERVER_PLANETSCENTRAL_DBCLIENT_DBSET_H

#include "planetscentral/dbclient/dbkey.h"
#include "planetscentral/dbclient/connection.h"
#include "planetscentral/dbclient/dbsort.h"

/** A set key. Contains an unordered set of items. Use the derived classes
    DbIntSet and DbStringSet for operations working on the items. */
class DbSet : public DbKey {
 public:
    DbSet(DbClient& client, const string_t& name);
    int32_t size();
    bool empty();
    DbSort sort();
};

class DbIntSetOperation;
class DbStringSetOperation;

/** Set of integers. */
class DbIntSet : public DbSet {
 public:
    using DbKey::remove;

    DbIntSet(DbClient& client, const string_t& name);
    bool add(int32_t value);
    bool remove(int32_t value);
    int32_t extractRandom();
    int32_t getRandom();
    bool moveTo(int32_t value, const DbIntSet& other);
    bool contains(int32_t value);
    void getAll(IntList_t& list);
    DbIntSetOperation intersect(DbIntSet other);
    DbIntSetOperation merge(DbIntSet other);
    DbIntSetOperation remove(DbIntSet other);
};

/** Set of strings. */
class DbStringSet : public DbSet {
 public:
    using DbKey::remove;

    DbStringSet(DbClient& client, const string_t& name);
    bool add(string_t value);
    bool remove(string_t value);
    string_t extractRandom();
    string_t getRandom();
    bool moveTo(string_t value, const DbStringSet& other);
    bool contains(string_t value);
    void getAll(StringList_t& list);
    DbStringSetOperation intersect(DbStringSet other);
    DbStringSetOperation merge(DbStringSet other);
    DbStringSetOperation remove(DbStringSet other);
};


/***************************** Set Operations ****************************/

class DbIntSetOperation {
 public:
    DbIntSetOperation(const char* op, DbIntSet a, DbIntSet b);
    DbIntSetOperation& andAlso(DbIntSet x);
    void get(IntList_t& list);
    void storeTo(DbIntSet other);

 private:
    DbClient& client;
    Ptr<DbRequest> req;
};

class DbStringSetOperation {
 public:
    DbStringSetOperation(const char* op, DbStringSet a, DbStringSet b);
    DbStringSetOperation& andAlso(DbStringSet x);
    void get(StringList_t& list);
    void storeTo(DbStringSet other);

 private:
    DbClient& client;
    Ptr<DbRequest> req;
};

#endif
