/**
  *  \file planetscentral/osutil.cc
  *  \brief Operating System Support
  */

#define _XOPEN_SOURCE 500

#include <stdexcept>
#include "planetscentral/osutil.h"

/******************************* Semaphore *******************************/

Semaphore::Semaphore(int value)
{
    if (sem_init(&sem, 0, value) < 0) {
        throw std::runtime_error("unable to create semaphore");
    }
}

Semaphore::~Semaphore()
{
    sem_destroy(&sem);
}

void
Semaphore::up()
{
    sem_post(&sem);
}

void
Semaphore::down()
{
    sem_wait(&sem);
}

void
Semaphore::timedDown(time_t time)
{
    struct timespec ts;
    ts.tv_sec = time;
    ts.tv_nsec = 0;
    sem_timedwait(&sem, &ts);
}

/********************************* Mutex *********************************/

Mutex::Mutex()
{
    pthread_mutexattr_t attr;
    if (pthread_mutexattr_init(&attr) != 0
        || pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE) != 0
        || pthread_mutex_init(&mtx, &attr) != 0
        || pthread_mutexattr_destroy(&attr) != 0)
    {
        throw std::runtime_error("unable to create mutex");
    }
}

Mutex::~Mutex()
{
    pthread_mutex_destroy(&mtx);
}

void
Mutex::enter()
{
    pthread_mutex_lock(&mtx);
}

void
Mutex::leave()
{
    pthread_mutex_unlock(&mtx);
}
